(function (global) {
    "use strict";

    global.FRCC = global.FRCC || {};
    global.FRCC.Portal = global.FRCC.Portal || {};
	global.FRCC.Portal.Documents = (function () {

		class DocumentContext {
            constructor(documentPath, idTag) {
                this.documentPath = documentPath;
				this.idTag = idTag;
            }
		}

		var _getDocumentContextFromForm = function() {
			var documentPath = $("#hdn-documentpath").val();
			var applicationNumber = $("#hdn-applicationnumber").val();
			return new DocumentContext(documentPath, applicationNumber)
		}

		var _getEygaApiContextFromForm = function () {
			var regardingId = $("#hdn-recordid").val();
			var regardingType = $("#hdn-recordtype").val();
			var userId = $("#hdn-userid").val();
			return new FRCC.Portal.Utilities.EygaApiContext(regardingId, regardingType, userId)
		}

		var _getWebResourceId = (docUploadId) => `WebResource_FileUpload_${docUploadId}`;
		var _getSessionName = (docUploadId) => `Files_${docUploadId}`;

		var setupFileUploads = (config, childId = null, domContext = document, pageValidators = null) => {
			var childId = childId;
			var domContext = domContext;
			config.forEach(docUpload => {
				var webResourceId = _getWebResourceId(docUpload.Identifier);
				ConvertIframeIntoElements(webResourceId);
				$(`#${webResourceId} .file-upload-core`, domContext).change(function () {
					_uploadFiles(
						docUpload.Identifier,
						docUpload.FolderName,
						$(this)[0].files,
						docUpload.DocumentValidation,
						childId,
						domContext);
					$(this)[0].value = null;
				});
				
				//$(`#${webResourceId} p`).attr({'aria-atomic':'true', 'aria-live': 'assertive'}); // Needed for accessibility
				$(`#${webResourceId} .alert-success`).attr({'aria-label':'success'}); // Needed for accessibility
				//$(`#${webResourceId} .disable-upload-message`, domContext).hide();
				//$(`#${webResourceId} .upload-button`, domContext).removeAttr('disabled');
				//$(`#${webResourceId} .table`, domContext).show();

				_listFilesById(docUpload.Identifier, docUpload.FolderName,
					_getEygaApiContextFromForm(), _getDocumentContextFromForm(), childId, domContext);
				_setupFileUploadValidation(docUpload, domContext, pageValidators || Page_Validators);
			});
		};

		var ConvertIframeIntoElements = (webResourceId) =>
		{
			var control = $('#' + webResourceId);
			if( control.length > 0 && control[0].tagName == "IFRAME")
			{
				var uploadControlBody = $(control).contents().find("body").html();
				$(control).parent().append("<literal id='"+webResourceId+"'>"+uploadControlBody+"</literal>");
				$(control).remove();
			}
		};

		var downloadAttachment = async(fullName) => {
			var eygaApiContext = _getEygaApiContextFromForm();
			var payload = fullName;

			var result = await global.FRCC.Portal.Utilities.callEygaApi(
				FRCC.Portal.Utilities.ApiCall.DownloadDocument, eygaApiContext, payload);

			var blob = new Blob([result], { type: "application/octet-stream" });
			var link = document.createElement('a');
			link.href = window.URL.createObjectURL(blob);
			link.download = fullName.split("/").pop();
			link.click();
        };

		var _uploadFiles = async (docUploadId, folderName, fileArr,
			documentValidation, childId = null, domContext = document) => {
            try {
				var webResourceId = _getWebResourceId(docUploadId);
				$(`#${webResourceId} .alert-success`, domContext).hide();
				$(`#${webResourceId} .alert-success`).attr({'aria-hidden':'true'}); // Needed for accessibility
				$(`#${webResourceId} .alert-success`).removeAttr('aria-live aria-atomic');
				$(`#${webResourceId} .loading-ui`, domContext).find('p').text('Uploading files');
				$(`#${webResourceId} .loading-ui`, domContext).attr('role', 'alert')
				$(`#${webResourceId} .loading-ui`, domContext).show();

                console.log("Uploading files...");

				var file = fileArr[0];

				var validationError = _isFileValid(file);
				if (validationError) {
					throw validationError;
				}
				//TODO: IF HEIC file type convert first to jpeg
				
				var eygaApiContext = _getEygaApiContextFromForm();
				var documentContext = _getDocumentContextFromForm();

				if (documentValidation == "203300001") { // Only one doc of type
					var existingFiles = global.FRCC.Portal.Utilities.getSession(_getSessionName(docUploadId));
					if (existingFiles?.[folderName]?.length) {
						for (var i = 0; i < existingFiles[folderName].length; i++) {
							await _deleteFile(eygaApiContext, existingFiles[folderName][i]);
                        }
                    }
				}

				await _uploadFile(eygaApiContext, documentContext, file, folderName, childId);

				setTimeout(() => {
					_listFilesById(docUploadId, folderName, eygaApiContext, documentContext, childId, domContext);
					$(`#${webResourceId} .loading-ui`, domContext).hide();
					$(`#${webResourceId} .loading-ui`, domContext).removeAttr('role');
					$(`#${webResourceId} .alert-success`, domContext).show();
					$(`#${webResourceId} .alert-success`).attr({'aria-atomic':'true', 'aria-live': 'polite'}); // Needed for accessibility
				}, 1000)
			} catch (error) {
				alert(error);
				console.log(error);
				$(`#${webResourceId} .loading-ui`, domContext).hide();
				$(`#${webResourceId} .loading-ui`, domContext).removeAttr('role');
			}
        };

		var _uploadFile = async (eygaApiContext, documentContext, file, folderName, childId = null) => {
			var payload = new FormData();
			var tags = {
				"Id": documentContext.idTag,
				"Folder": folderName,
				"Upload Date": new Date().toLocaleString().replace(/,/g, '').replace(/:/g, '-'),
				"Status": "Accepted",
				"RestrictDelete": false
			};
			if (childId) { tags["ChildId"] = childId; }
			
			payload.append('json', JSON.stringify({
				"Name": sanitizeFileName(file.name),
				"DocumentFolder": folderName,
				"RecordFolder": documentContext.documentPath,
				"Tags": tags
			}));
			payload.append('file', file);

			await global.FRCC.Portal.Utilities.callEygaApi(
				FRCC.Portal.Utilities.ApiCall.UploadDocument, eygaApiContext, payload);
		}

		var _listFilesById = async (docUploadId, folderName, eygaApiContext, documentContext, childId = null, domContext = document) => {
			var webResourceId = _getWebResourceId(docUploadId);
			$(`#${webResourceId} .loading-ui`, domContext).find('p').text('Loading files');

			var payload = {
				"Id": documentContext.idTag,
				"Folder": folderName,
				"Status": "Accepted"
			};
			if (childId) { payload["ChildId"] = childId; }
			var response = await global.FRCC.Portal.Utilities.callEygaApi(
				FRCC.Portal.Utilities.ApiCall.GetDocumentsById, eygaApiContext, payload);

			if (response?.length) {
				$(`#${webResourceId} .table`, domContext).show();
			}

			$(`#${webResourceId} .table`, domContext).find("tr:gt(0)").remove();
			var files = {};
			for (const file of response) {
				var newRow = domContext.createElement('tr');
				var nameCell = domContext.createElement('td');
				nameCell.innerHTML = `<button type="button" class="btn btn-link-inline" href="javascript:void(0)" onclick="FRCC.Portal.Documents.downloadAttachment('${file.fullName}')"><p>${file.name}</p></button>`;
				newRow.appendChild(nameCell);

				var deleteCell = domContext.createElement('td');
				deleteCell.className = "contains-component text-right";
				var deleteLink = domContext.createElement('button');
				deleteLink.href = "javascript:void(0)";
				deleteLink.type = "button";
				deleteLink.className = "btn btn-link btn-md btn-icon-only";
				deleteLink.ariaLabel = "remove";
				deleteLink.onclick = () => {
					global.FRCC.Portal.Utilities.openConfirmationModal(
						"Delete File",
						"Are you sure you want to remove this file?",
						"Delete")
						.then(async (result) => {
							if (!result) { return; }

							try {
								$(`#${webResourceId} .alert-success`, domContext).hide();
								$(`#${webResourceId} .alert-success`).attr({'aria-hidden':'true'}); // Needed for accessibility
								$(`#${webResourceId} .alert-success`).removeAttr('aria-live').removeAttr('aria-atomic');
								$(`#${webResourceId} .loading-ui`, domContext).find('p').text('Deleting files...');
								$(`#${webResourceId} .loading-ui`, domContext).attr('role','alert');
								$(`#${webResourceId} .loading-ui`, domContext).show();

								console.log("Deleting file...");

								await _deleteFile(eygaApiContext, file.fullName);

								setTimeout(() => {
									_listFilesById(docUploadId, folderName, eygaApiContext, documentContext, childId, domContext);
									$(`#${webResourceId} .loading-ui`, domContext).hide();
									$(`#${webResourceId} .loading-ui`, domContext).removeAttr('role');
								}, 1000);
							} catch (error) {
								alert(error);
								console.log(error);
								$(`#${webResourceId} .loading-ui`, domContext).hide();
								$(`#${webResourceId} .loading-ui`, domContext).removeAttr('role');
							}
						});
				};
				deleteLink.innerHTML = `<span class="glyphicon glyphicon-trash icon-size-md"></span>`;
				deleteCell.appendChild(deleteLink);
				newRow.appendChild(deleteCell);

				$(newRow).insertAfter($(`#${webResourceId} .table tr:last`, domContext));

				if (!files[file.tags.Folder]) { files[file.tags.Folder] = []; }
				files[file.tags.Folder].push(file.fullName);
			}

			global.FRCC.Portal.Utilities.saveSession(_getSessionName(docUploadId), files);

			$(`#${webResourceId} .loading-ui`, domContext).hide();
			$(`#${webResourceId} .loading-ui`, domContext).removeAttr('role');
		};

		var _deleteFile = async(eygaApiContext, fullName) => {
		    
			var payload = fullName;
			return global.FRCC.Portal.Utilities.callEygaApi(
				FRCC.Portal.Utilities.ApiCall.DeleteDocument, eygaApiContext, payload);
		};

		var _setupFileUploadValidation = function(docUpload, domContext = document, pageValidators = null) {
			var webResourceId = _getWebResourceId(docUpload.Identifier);
			var target = $(`#${webResourceId} .form-field-file-upload`, domContext).parents('.control').siblings('.info');
			switch (docUpload.DocumentValidation) {
				case 203300001:
					target.addClass('required');
					_addPageValidator(docUpload, () => {
						var files = global.FRCC.Portal.Utilities.getSession(_getSessionName(docUpload.Identifier));
						var folderFiles = files?.[docUpload.FolderName];

						if(folderFiles?.length){
							target.removeClass('has-error');
							return true;
						}
						else{
							target.addClass('has-error');
							return false;
						}
					},
					`You must attach one required document for this category: ${docUpload.FolderName}`,
					"You can only submit one document. Any new document will replace the existing one.",
					domContext, pageValidators || Page_Validators);
					break;
				case 203300002:
					target.addClass('required');
					_addPageValidator(docUpload, () => {
						var files = global.FRCC.Portal.Utilities.getSession(_getSessionName(docUpload.Identifier));
						var folderNames = Object.keys(files);

						if(folderNames?.includes(docUpload.FolderName)){
							target.removeClass('has-error');
							return true;
						}
						else{
							target.addClass('has-error');
							return false;
						}
					},
					`You must attach one or more required documents for this category: ${docUpload.FolderName}`,
					null, domContext, pageValidators || Page_Validators);
            }
		};

		var _addPageValidator = function (docUpload, validationFunction, validationMessage,
			tooltipMessage = null, domContext = document, pageValidators = null) {
			if (tooltipMessage) {
				var webResourceId = _getWebResourceId(docUpload.Identifier);
				$(`#${webResourceId} .file-upload-core`, domContext).attr("title", tooltipMessage);
				$(`#${webResourceId} .file-upload-core`, domContext).attr("data-toggle", "tooltip");
            }

			if (!pageValidators) { pageValidators = Page_Validators; }
			if (typeof (pageValidators) == 'undefined') { return; }
			var validator = document.createElement('span');
			validator.style.display = "none";
			validator.id = `validator-${docUpload.Identifier}`;
			validator.controltovalidate = "";
			validator.errormessage = `<a href='#WebResource_FileUpload_${docUpload.Identifier}'>${validationMessage}</a>`;
			validator.validationGroup = `validation-${docUpload.StepNumber}`;
			validator.initialvalue = "";
			validator.evaluationfunction = validationFunction;
			pageValidators.push(validator);
		};

        var _isFileValid = function(file) {
			var cleanFileName = sanitizeFileName(file.name);
            var fileExtension = cleanFileName.split(".")[1];
            var validExtensions = ['application/vnd.openxmlformats-officedocument.wordprocessingml.document', 'image/bmp', 'application/msword', 'application/pdf', 'application/vnd.openxmlformats-officedocument.wordprocessingm', 'image/gif', 'image/jpeg', 'image/png', 'application/vnd.ms-powerpoint',
              'application/vnd.openxmlformats-officedocument.presentationml.presentation', 'application/rtf', 'image/tiff', 'text/plain', 'application/vnd.ms-excel', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
              'image/heic', 'image/heif', 'text/csv'];
    
            if (!validExtensions.includes(file.type)) {
				//sometimes heic files have empty file types and should not go into this alert
				if (fileExtension !== "heic" && fileExtension !== "heif") {
					return "ERROR: You are attempting to upload an invalid document type.";
				}
            }
            //throw error if file size is greater than 100mb
            if (file.size/1000000 > 100) {
				return "ERROR: You are attempting to upload a file larger than 100mb";
			}

			return null;
        }

		function sanitizeFileName(fileName) {
            // Preserve letters, numbers, and allowed special characters (_, -, .)
            // Replace all other special characters with underscore
            return fileName.replace(/[^a-zA-Z0-9._-]/g, '_');
        }

		return {
			setupFileUploads: setupFileUploads,
            downloadAttachment: downloadAttachment,
			DocumentContext: DocumentContext
        };
    })();

})(this);